#!/bin/bash

DIRECTORY=/data/sensor-data
LOG_FILE=$DIRECTORY/sd_upgrade.log
HOST_DATA_DIR_CONFIG=$DIRECTORY/config
HOST_DATA_DIR_DB=$DIRECTORY/db
HOST_DATA_DIR_LOGS=$DIRECTORY/logs
CONTAINER_NAME=sensor-data
APP_VERSION=max1.0-1.0.1
MIRROR=vofo
APP_IMAGE=$MIRROR/$CONTAINER_NAME:$APP_VERSION

PORT=6002

preCheck() {
    
    if ! command -v tee > /dev/null; then
        echo "`date` tee command not found. Please install tee." >> $LOG_FILE
        exit 1
    fi

    if ! command -v curl > /dev/null; then
        echo "`date` curl command not found. Please install curl." | tee -a $LOG_FILE
        exit 1
    fi

    if ! command -v docker-compose > /dev/null; then
        echo "`date` docker-compose command not found. Please install docker-compose." | tee -a $LOG_FILE
        exit 1
    fi
    
    docker pull $APP_IMAGE
    pullResult=$?
    if [ $pullResult -ne 0 ]; then
        echo "`date` pull $CONTAINER_NAME image failed with exit code $pullResult" | tee -a $LOG_FILE
        return 1
    fi

    # Check sensor-data status
    if docker ps -a | grep $CONTAINER_NAME | grep Up > /dev/null; then
        contain_id=`docker ps | grep $CONTAINER_NAME | awk '{print $1}'`
        docker stop $contain_id
        stopResult=$?
        if [ $stopResult -ne 0 ]; then
            echo "`date` stop $contain_id failed with exit code $stopResult" | tee -a $LOG_FILE
            return 1
        fi
        docker rm $contain_id
        rmResult=$?
        if [ $rmResult -ne 0 ]; then
            echo "`date` rm $contain_id failed with exit code $rmResult" | tee -a $LOG_FILE
            return 1
        fi
    fi

    if docker ps -a | grep $CONTAINER_NAME | grep Exit > /dev/null; then
        contain_id=`docker ps -a | grep $CONTAINER_NAME | awk '{print $1}'`
        docker rm $contain_id
        rmResult=$?
        if [ $rmResult -ne 0 ]; then
            echo "`date` rm $contain_id failed with exit code $rmResult" | tee -a $LOG_FILE
            return 1
        fi
    fi
    
    
    cp -f ./default.json $DIRECTORY/config/default.json
    
    return 0
}

upgrade() {
    
    echo "`date` start upgrade $CONTAINER_NAME on port $PORT" | tee -a $LOG_FILE
    # Create docker-compose.yml
    echo "version: '3'" > docker-compose.yml
    echo "services:" >> docker-compose.yml
    echo "  $CONTAINER_NAME:" >> docker-compose.yml
    echo "    image: $APP_IMAGE" >> docker-compose.yml
    echo "    container_name: $CONTAINER_NAME" >> docker-compose.yml
    echo "    restart: unless-stopped" >> docker-compose.yml
    echo "    network_mode: host" >> docker-compose.yml
    echo "    volumes:" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_CONFIG:/opt/sensor-data/config" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_DB:/opt/sensor-data/.config/foggie/db" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_LOGS:/opt/sensor-data/.config/foggie/logs" >> docker-compose.yml
    
    # Start sensor-data
    docker-compose up -d
    upResult=$?
    if [ $upResult -ne 0 ]; then
        echo "`date` upgrade $CONTAINER_NAME failed retry 1" | tee -a $LOG_FILE
        sleep 1
        docker-compose up -d
        upRetryOneResult=$?
        if [ $upRetryOneResult -ne 0 ]; then
            echo "`date` upgrade $CONTAINER_NAME failed retry 2" | tee -a $LOG_FILE
            sleep 1
            docker-compose up -d
            upRetryTwoResult=$?
            if [ $upRetryTwoResult -ne 0 ]; then
                echo "`date` upgrade $CONTAINER_NAME failed with exit code $upRetryTwoResult" | tee -a $LOG_FILE
                return 1
            fi
        fi
    fi
    echo "`date` wait 10 seconds for $CONTAINER_NAME to start" | tee -a $LOG_FILE
    sleep 10
    
    return 0
}

postCheck() {
    echo "`date` start post check for $CONTAINER_NAME upgrade" | tee -a $LOG_FILE
    # Check sensor-data status
    if ! docker ps | grep $CONTAINER_NAME | grep Up > /dev/null; then
        echo "`date` sensor-data is not running." | tee -a $LOG_FILE
        return 1
    fi

    image_ids=$(docker images | grep $CONTAINER_NAME | awk '{print $3}')
    # 删除这些无标签的镜像
    for id in $image_ids; do
        echo "`date` Deleting image with ID $id" | tee -a $LOG_FILE
        docker rmi $id
    done
    return 0
}

# step 1: pre-Check
preCheck
preCheckResult=$?
echo "`date` pre check result is $preCheckResult" | tee -a $LOG_FILE
if [ $preCheckResult -eq 0 ]
then
    echo "`date` Pre-check successful." | tee -a $LOG_FILE
    # Step 2: Upgrade sensor-data
    upgrade $1
    upgradeResult=$?
    if [ $upgradeResult -eq 0 ]
    then
        echo "`date` Upgrade sensor-data successful." | tee -a $LOG_FILE
        # Step 3: Post-check
        postCheck
        postCheckResult=$?
        if [ $postCheckResult -eq 0 ]
        then
            echo "`date` Post-check successful." | tee -a $LOG_FILE
            echo "`date` sensor-data upgrade process completed." | tee -a $LOG_FILE
            exit 0
        else
            echo "`date` Post-check failed." | tee -a $LOG_FILE
            exit 6
        fi
    else
        echo "`date` Upgrade sensor-data failed." | tee -a $LOG_FILE
        exit 5
    fi
else
    echo "`date` Pre-check failed." | tee -a $LOG_FILE
    exit 4
fi